/*
 *  sentoria.c - create sparse bitmap using "text" screen and charset up to 256 8x8 characters.
 *
 *        creation date: 2015-12-25  Michael Kircher
 *  date of last change: 2018-04-14  Michael Kircher
 */

#include <stdio.h>
#include <stdlib.h>
#include "image.h"
#include "matrix.h"

#define START_CHAR 2

TEMPLATE_MATRIX(byte,unsigned char)

typedef enum {false,true} boolean;

int main(int argc, char *argv[])
{
  int cols,rows; /* dimensions of text screen */
  int x,y,y2;
  int i,j,max;
  unsigned char charset[256][8];
  unsigned char **screen;
  IMAGE *src;
  FILE *stream;

  if(       NULL == (src=image_load("input"))
     || BYTE_IMG != src->type
     ||        0 != src->rows%8
     ||        0 != src->cols%8) exit(EXIT_FAILURE);

  rows=src->rows/8;
  cols=src->cols/8;

  if(NULL==(screen=byte_alloc_matrix(rows,cols)))
  {
    image_free(src);
    exit(EXIT_FAILURE);
  }

  max=START_CHAR;
  for(y=0; y<rows; y++)
    for(x=0; x<cols; x++)
    {
      unsigned char glyph[8];
      /* extract bitmap data into 8 bytes */
      for(y2=0; y2<8; y2++)
        glyph[y2]=128*(b_pixel(src,x*8+0,y*8+y2) != 0)
                 + 64*(b_pixel(src,x*8+1,y*8+y2) != 0)
                 + 32*(b_pixel(src,x*8+2,y*8+y2) != 0)
                 + 16*(b_pixel(src,x*8+3,y*8+y2) != 0)
                 +  8*(b_pixel(src,x*8+4,y*8+y2) != 0)
                 +  4*(b_pixel(src,x*8+5,y*8+y2) != 0)
                 +  2*(b_pixel(src,x*8+6,y*8+y2) != 0)
                 +  1*(b_pixel(src,x*8+7,y*8+y2) != 0);

      /* match against all current glyphs in character set: */
      j=max; /* default for non-match */
      for(i=START_CHAR; i<max; i++)
        if(   glyph[0] == charset[i][0]
           && glyph[1] == charset[i][1]
           && glyph[2] == charset[i][2]
           && glyph[3] == charset[i][3]
           && glyph[4] == charset[i][4]
           && glyph[5] == charset[i][5]
           && glyph[6] == charset[i][6]
           && glyph[7] == charset[i][7]) {j=i; break;}

      screen[y][x]=j; /* fan-out to screen */

      /* add glyph to character set if no duplicate was found: */
      if(j == max && max<256)
      {
        for(i=0; i<8; i++)
          charset[j][i]=glyph[i];
        max++;
      }
    }
  
  /* write out screen binary */
  if(NULL==(stream=fopen("screen.bin","wb")))
  {
    byte_free_matrix(screen);
    image_free(src);
    exit(EXIT_FAILURE);
  }
  for(y=0; y<rows; y++)
    for(x=0; x<cols; x++)
      fputc(screen[y][x],stream);
  fclose(stream);

  /* write out character set binary */
  if(NULL==(stream=fopen("charset.bin","wb")))
  {
    byte_free_matrix(screen);
    image_free(src);
    exit(EXIT_FAILURE);
  }
  for(j=START_CHAR; j<max; j++)
    for(i=0; i<8; i++)
      fputc(charset[j][i],stream);
  fclose(stream);

  byte_free_matrix(screen);
  image_free(src);

  exit(EXIT_SUCCESS);
}